<?php

namespace app\models;

use Yii;

/**
 * This is the model class for table "Obra".
 *
 * @property string $id
 * @property string $nombre
 * @property string|null $fechaIncidente
 * @property string|null $idObra
 * @property string|null $idUsuario
 * @property string|null $descripcion
 * @property string|null $creado
 * @property string|null $modificado
 * @property string|null $eliminado
 *
 * @property IncidenteEmpleado[] $incidenteEmpleado
 * @property Empleado $idEmpleado
 */
class Incidente extends ModeloBase
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'Incidente';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'nombre', 'idObra'], 'required'],
            [['fechaIncidente','creado', 'modificado', 'eliminado'], 'safe'],
            [['id', 'idObra','idUsuario'], 'string', 'max' => 36],
            [['nombre', 'descripcion'], 'string', 'max' => 100],
            [['id'], 'unique'],
            [['idObra'], 'exist', 'skipOnError' => true, 'targetClass' => Obra::class, 'targetAttribute' => ['idObra' => 'id']],
            [['idUsuario'], 'exist', 'skipOnError' => true, 'targetClass' => Usuario::class, 'targetAttribute' => ['idUsuario' => 'id']],

        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'nombre' => 'Nombre',
            'fechaIncidente' => 'Fecha Incidente',
            'idObra' => 'ID Obra',
            'idUsuario' => 'Id Usuario',
            'descripcion' => 'Descripcion',
            'creado' => 'Creado',
            'modificado' => 'Modificado',
            'eliminado' => 'Eliminado',
        ];
    }

   

    public function getIncidenteEmpleado()
    {
        return $this->hasMany(IncidenteEmpleado::class, ['idIncidente' => 'id']);
    }

    public function getObra()
    {
        return $this->hasOne(Obra::class, ['id' => 'idObra']);
    }
    public function getUsuario()
    {
        return $this->hasOne(Usuario::class, ['id' => 'idUsuario']);
    }
}
