<?php

namespace v1\controllers;

use app\models\Media;
use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class CampanaController extends AuthController {

  public $modelClass = "v1\models\Campana";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial;

    if ($id > 0) {
      $query->andWhere(["id" => $id]);
    }

    if ($buscar) {

      $query->andWhere([
        "OR",
        "f_unaccent([[nombre]]) ilike f_unaccent(:q)",
      ])->addParams([':q' => "%{$buscar}%"]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $archivos = $this->req->getBodyParam("archivos", []);
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }
    if ($modelo === null) {
      $modelo = new $this->modelClass();
      $modelo->uuid();
      $modelo->creado = new Expression('now()');
    } else {
      $modelo->modificado = new Expression('now()');
    }

    $modelo->load($this->req->getBodyParams(), '');

    if (empty($archivos)) {
      $modelo->idMedia = null;
    } else {
      $modelo->idMedia = $archivos[0]['id'];
    }

    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("Hubo un problema al guardar la Campaña");
    }

    $modelo->refresh();
    return (new Respuesta($modelo))
      ->mensaje("Campaña guardada");
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Campaña no encontrada");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar la Campaña");
    }

    return (new Respuesta())
      ->mensaje("Campaña eliminada");
  }
}
