<?php

namespace v1\controllers;

use app\models\ObraEmpleado;
use app\models\ObraHerramienta;
use common\data\Respuesta;
use common\rest\AuthController;
use Exception;
use Mpdf\Tag\Select;
use Yii;
use yii\db\Expression;
use yii\db\Query;
class ObraController extends AuthController {

  public $modelClass = "v1\models\Obra";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $inicio = trim($this->req->get("inicio", ""));
    $fin = trim($this->req->get("fin", ""));
   
    $query = $this->queryInicial;

    if ($id!=="") {
      $query->andWhere(["id" => $id]);
    }

    if ($inicio !== "" && $fin !== "") {
      $query->andWhere([">=", "[[fechaInicio]]", $inicio])
            ->andWhere(["<=", "[[fechaFinal]]", $fin]);
  }
    if ($buscar) {

      $query->andWhere([
        "OR",
        ["ilike", "nombre", $buscar]
      ]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $herramientas = $this->req->getBodyParam("herramientas", []);
    $empleados = $this->req->getBodyParam("empleados", []);
    $modelo = null;

    $transaccion = \Yii::$app->db->beginTransaction();
    try {

      if ($id !== "") {
        $modelo = $this->modelClass::findOne($id);
      }
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }
      $modelo->load($this->req->getBodyParams(), '');

      $clave = $modelo->clave;
      $existeClave = $this->modelClass::find()->andwhere(['clave' => $clave])->andWhere(['<>', 'id', $id])->andWhere(['eliminado'=>null])->exists();

      if ($existeClave) {
        return (new Respuesta())
          ->esError()
          ->mensaje("La clave '$clave' ya existe. Por favor, ingresa una clave diferente.");
      }
      $modelo->load($this->req->getBodyParams(), '');
      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar la obra");
      }

     

      ObraHerramienta::deleteAll(['idObra' => $modelo->id]);

      foreach ($herramientas as $herramienta) {
        if (isset($herramienta['id']) && isset($herramienta['cantidad'])) {
          $obraHerramienta = new ObraHerramienta();
          $obraHerramienta->uuid();
          $obraHerramienta->idHerramienta = $herramienta['id'];
          $obraHerramienta->idObra = $modelo->id;
          $obraHerramienta->cantidad = $herramienta['cantidad'];

          if (!$obraHerramienta->save()) {
            $transaccion->rollBack();
            return (new Respuesta($obraHerramienta))
              ->mensaje("Hubo un problema al guardar la Herramienta con ID {$herramienta['id']}");
          }
        } else {
          Yii::error('Faltan datos en el array de herramientas', ['item' => $herramienta]);
        }
      }

      $transaccion->commit();
      $modelo->refresh();
      return (new Respuesta($modelo))
        ->mensaje("Obra ha sido guardado de manera exitosa");
    } catch (\Exception $e) {
      $transaccion->rollBack();
      return (new Respuesta($modelo))
        ->esError()
        ->mensaje($e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Obra no encontrada");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar la obra");
    }

    return (new Respuesta())
      ->mensaje("Obra eliminada");
  }

  public function actionTotalGasto($id){
    $TotalGasto=0;
    $q=(new Query())->select(['{{Gasto}}.[[cantidad]]'])
    ->from('Gasto') 
    ->innerJoin('Obra', '{{Obra}}.[[id]] = {{Gasto}}.[[idObra]]') 
    ->andWhere(['{{Gasto}}.[[idObra]]' => $id]) 
    ->andWhere(['{{Gasto}}.[[eliminado]]'=>null]) 
    ->column();
    
    foreach($q as $cantidad){
      $TotalGasto=$TotalGasto+$cantidad;
    }
    return $TotalGasto;
  }

  public function actionTotalPago($id){
    $TotalPago= 0;
    $q=(new Query())->select(['{{Pago}}.[[cantidad]]'])
    ->from('Pago') 
    ->innerJoin('Obra', '{{Obra}}.[[id]] = {{Pago}}.[[idObra]]') 
    ->andWhere(['{{Pago}}.[[idObra]]' => $id]) 
    ->andWhere(['{{Pago}}.[[eliminado]]'=>null]) 
    ->column();
    foreach($q as $cantidad){
      $TotalPago=$TotalPago+$cantidad;
    }
    return $TotalPago;
  }

  public function actionTotalNomina($id){
    try {
    $TotalNomina= 0;
    $NominaDescuento=0;
    $q=(new Query())->select(['{{Nomina}}.[[montoPagado]]','{{Nomina}}.[[descuento]]'])
    ->from('Nomina') 
    ->innerJoin('Obra', '{{Obra}}.[[id]] = {{Nomina}}.[[idObra]]') 
    ->andWhere(['{{Nomina}}.[[idObra]]' => $id]) 
    ->andWhere(['{{Nomina}}.[[eliminado]]'=>null])
    ->all();
    foreach($q as $cantidad){
      $montoPagado= $cantidad['montoPagado'];
      $descuento= $cantidad['descuento'];
      $NominaDescuento= $montoPagado-($montoPagado*($descuento/100));
      $TotalNomina=$TotalNomina+$NominaDescuento;
    }
    return $TotalNomina;
  } catch (\Exception $e) {
    Yii::error("Error en actionTotalNomina: " . $e->getMessage());
    return ['error' => 'Error al calcular la nómina. Verifique el log para más detalles.'];
}
}

public function actionTotalRubro($id){
  try {
  $TotalRubro= 0;
  $q=(new Query())->select(['{{ObraRubro}}.[[cantidad]]', '{{ObraRubro}}.[[descuento]]'])
  ->from('ObraRubro') 
  ->rightJoin('Obra', '{{Obra}}.[[id]] = {{ObraRubro}}.[[idObra]]') 
  ->andWhere(['{{ObraRubro}}.[[idObra]]' => $id]) 
  ->andWhere(['{{ObraRubro}}.[[eliminado]]'=>null])
  ->all();
  foreach ($q as $rubro) {
    $cantidad = $rubro['cantidad'];
    $descuento = $rubro['descuento'] ?? 0; 

    $cantidadConDescuento = $cantidad * (1 - $descuento / 100);

    // Suma al total
    $TotalRubro += $cantidadConDescuento;
}
  return $TotalRubro;
} catch (\Exception $e) {
  Yii::error("Error en actionRubro: " . $e->getMessage());
  return ['error' => 'Error al calcular el Rubro. Verifique el log para más detalles.'];
}
}


public function actionSaldo($gasto, $nomina, $pago){
 try{
  $TotalSaldo=($gasto+$nomina)-$pago;
  return $TotalSaldo; 

 }catch(\Exception $e){
  Yii::error("Error en actionsaldo: " . $e->getMessage());
  return ['error' => 'Error al calcular el Saldo. Verifique el log para más detalles.'];
 } 
}

public function actionEjercidoObra($ejercer, $rubro){
  try{
    if (($ejercer - $rubro) != 0) {
      $TotalEjercido = ($rubro / ($ejercer - $rubro)) * 100;
      $TotalEjercido = round($TotalEjercido, 2);

  } else {
      $TotalEjercido = 0; // Evita dividir por cero
  }

   return $TotalEjercido; 
 
  }catch(\Exception $e){
   Yii::error("Error en actionEjercidoObra: " . $e->getMessage());
   return ['error' => 'Error al calcular el Ejercido. Verifique el log para más detalles.'];
  } 
 }
 public function actionEjercer($gasto, $nomina, $rubro){
  try{
   $TotalEjercer=($gasto+$nomina)-$rubro;
   return $TotalEjercer; 
 
  }catch(\Exception $e){
   Yii::error("Error en actionEjercidoObra: " . $e->getMessage());
   return ['error' => 'Error al calcular el Ejercido. Verifique el log para más detalles.'];
  } 
 }
}

